module alu
  (
    // Outputs
    out_result, out_branch,
    // Inputs
    inp_a, inp_b, inp_instruction, inp_command
  );

  import types_pkg::*;

  // Inputs
  input  logic [WORD_BITS               -1:0] inp_a;
  input  logic [WORD_BITS               -1:0] inp_b;
  input  instruction_union_t                  inp_instruction;
  input  command_t                            inp_command;

  // Outputs
  output logic [WORD_BITS               -1:0] out_result;
  output logic                                out_branch;

  always_comb begin
    unique case ( inp_instruction.r.op )
      ADD     : out_result = inp_a + inp_b;
      SUB     : out_result = inp_a - inp_b;
      SRL     : out_result = inp_a >> 1;
      LUI     : out_result = { inp_instruction.i.x, 16'd0 };
      ADDI    : out_result = inp_a + { 16'd0, inp_instruction.i.x };
      SUBI    : out_result = inp_a - { 16'd0, inp_instruction.i.x };
      MULI    : out_result = inp_a[15:0] * inp_instruction.i.x;
      LW      : out_result = inp_a + { 16'd0, inp_instruction.i.x };
      LB      : out_result = inp_a + { 16'd0, inp_instruction.i.x };
      LFX     : out_result = inp_a + { 16'd0, inp_instruction.i.x };
      FADD    : out_result = { WORD_BITS { 1'bx } };
      FSUB    : out_result = { WORD_BITS { 1'bx } };
      FMUL    : out_result = { WORD_BITS { 1'bx } };
      BEQ     : out_result = { WORD_BITS { 1'bx } };
      BPOS    : out_result = { WORD_BITS { 1'bx } };
      BNEG    : out_result = { WORD_BITS { 1'bx } };
      BCEQ    : out_result = { WORD_BITS { 1'bx } };
      STAT    : out_result = { WORD_BITS { 1'bx } };
      POP     : out_result = { WORD_BITS { 1'bx } };
      PUSH    : out_result = { WORD_BITS { 1'bx } };
      FADDI   : out_result = { WORD_BITS { 1'bx } };
      FMULI   : out_result = { WORD_BITS { 1'bx } };
      ADDUI   : out_result = inp_a + { inp_instruction.i.x, 16'd0 };
      default : out_result = { WORD_BITS { 1'bx } };
    endcase
  end

  always_comb begin
    unique case ( inp_instruction.r.op )
      ADD     : out_branch = 1'b0 ;
      SUB     : out_branch = 1'b0 ;
      SRL     : out_branch = 1'b0 ;
      LUI     : out_branch = 1'b0 ;
      ADDI    : out_branch = 1'b0 ;
      SUBI    : out_branch = 1'b0 ;
      MULI    : out_branch = 1'b0 ;
      LW      : out_branch = 1'b0 ;
      LB      : out_branch = 1'b0 ;
      LFX     : out_branch = 1'b0 ;
      FADD    : out_branch = 1'b0 ;
      FSUB    : out_branch = 1'b0 ;
      FMUL    : out_branch = 1'b0 ;
      BEQ     : out_branch = inp_a == inp_b ;
      BPOS    : out_branch = inp_a[31] == 1'b0 ;
      BNEG    : out_branch = inp_a[31] == 1'b1 ;
      BCEQ    : out_branch = inp_command == inp_instruction.b.y ;
      STAT    : out_branch = 1'b0 ;
      POP     : out_branch = 1'b0 ;
      PUSH    : out_branch = 1'b0 ;
      FADDI   : out_branch = 1'b0 ;
      FMULI   : out_branch = 1'b0 ;
      ADDUI   : out_branch = 1'b0 ;
      default : out_branch = 1'b0 ;
    endcase
  end


endmodule