module drain
  (
    // Outputs
    slave_ready, master_valid, master_data, core_ready,
    // Inputs
    clk, reset_n, slave_valid, master_ready, slave_data, core_valid, core_data
  );

  import types_pkg::*;

  // Control
  input  logic                                reset_n;
  input  logic                                clk;

  // Slave interface
  input  logic                                slave_valid;
  output logic                                slave_ready;
  input  logic [RESULT_BITS             -1:0] slave_data;

  // Master interface
  output logic                                master_valid;
  input  logic                                master_ready;
  output logic [RESULT_BITS             -1:0] master_data;

  // Core interface
  input  logic                                core_valid;
  output logic                                core_ready;
  input  logic [RESULT_BITS             -1:0] core_data;

  // Internal signals
  logic [RESULT_BITS                    -1:0] tmp_nxt;
  logic [RESULT_BITS                    -1:0] tmp_r;
  logic                                       tmp_wren;
  logic                                       tmp_free_nxt;
  logic                                       tmp_free_r;
  logic                                       slave_accept;
  logic                                       master_accept;
  logic                                       core_accept;


  assign slave_accept = slave_ready & slave_valid;
  assign master_accept = master_ready & master_valid;
  assign core_accept = core_ready & core_valid;

  assign core_ready = (tmp_free_r | master_accept) & core_valid;
  assign slave_ready =(tmp_free_r | master_accept) & !core_valid;
  assign master_valid = !tmp_free_r;

  assign master_data = tmp_r;

  assign tmp_free_nxt = (master_accept & !(slave_accept | core_accept)) | (tmp_free_r & !(slave_accept | core_accept));
  always_ff @(posedge clk or negedge reset_n) begin
    if ( !reset_n ) begin
      tmp_free_r <= 1'b1;
    end
    else begin
      tmp_free_r <= tmp_free_nxt;
    end
  end

  assign tmp_nxt = (core_accept ? core_data : slave_data);
  assign tmp_wren = core_accept | slave_accept;
  always_ff @(posedge clk) begin
    if ( tmp_wren ) begin
      tmp_r <= tmp_nxt;
    end
  end

endmodule