module feeder_full
  (
    // Outputs
    slave_ready, master_valid, master_data, core_valid, core_data,
    // Inputs
    clk, reset_n, slave_valid, master_ready, slave_data, core_ready
  );

  import types_pkg::*;

  // Control
  input  logic                                reset_n;
  input  logic                                clk;

  // Slave interface
  input  logic                                slave_valid;
  output logic                                slave_ready;
  input  logic [REQUEST_BITS            -1:0] slave_data;

  // Master interface
  output logic                                master_valid;
  input  logic                                master_ready;
  output logic [REQUEST_BITS            -1:0] master_data;

  // Core interface
  output logic                                core_valid;
  input  logic                                core_ready;
  output logic [REQUEST_BITS            -1:0] core_data;


  // Internal signals
  logic                                master_valid_i;
  logic                                master_ready_i;
  logic [REQUEST_BITS            -1:0] master_data_i;

  feeder FEEDER (
      .reset_n(reset_n),
      .clk(clk),
      .slave_valid(slave_valid),
      .slave_ready(slave_ready),
      .slave_data(slave_data),
      .master_valid(master_valid_i),
      .master_ready(master_ready_i),
      .master_data(master_data_i),
      .core_valid(core_valid),
      .core_ready(core_ready),
      .core_data(core_data)
    );

  reverse_regslice #(
    .DATA_WIDTH(REQUEST_BITS)
  ) R_SLICE (
      .reset_n(reset_n),
      .clk(clk),
      .slave_valid(master_valid_i),
      .slave_ready(master_ready_i),
      .slave_data(master_data_i),
      .master_valid(master_valid),
      .master_ready(master_ready),
      .master_data(master_data)
  );

endmodule