module fpu
  (
    // Outputs
    out_result,
    // Inputs
    clk, inp_a, inp_b, inp_instruction
  );

  parameter ROUND = 0;

  import types_pkg::*;

  // Control
  // input  logic                                reset_n;
  input  logic                                clk;

  // Inputs
  input  logic [WORD_BITS               -1:0] inp_a;
  input  logic [WORD_BITS               -1:0] inp_b;
  input  instruction_union_t                  inp_instruction;

  // Outputs
  output logic [WORD_BITS               -1:0] out_result;

  // // Internal signals
  // shortreal                                   sr_res;
  // logic                                       clk_en;

  // always_comb begin
  //   clk_en = 1;
  //   case ( inp_op )
  //     FADD, FADDI, PUSH : sr_res = $bitstoshortreal(inp_a) + $bitstoshortreal(inp_b);
  //     FSUB              : sr_res = $bitstoshortreal(inp_a) - $bitstoshortreal(inp_b);
  //     FMUL, FMULI       : sr_res = $bitstoshortreal(inp_a) * $bitstoshortreal(inp_b);
      
  //     default     : begin
  //       sr_res = 0.0;
  //       clk_en = 0;
  //     end
  //   endcase
  // end

  // always_ff @(posedge clk) begin
  //   if ( clk_en ) begin
  //     out_result <= $shortrealtobits(sr_res);
  //   end
  // end

  logic [WORD_BITS                      -1:0] inp_b_muxed;
  always_comb begin
    unique case ( inp_instruction.r.op )
      ADD     : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      SUB     : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      SRL     : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      LUI     : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      ADDI    : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      SUBI    : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      MULI    : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      LW      : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      LB      : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      LFX     : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      FADD    : inp_b_muxed = inp_b ;
      FSUB    : inp_b_muxed = inp_b ;
      FMUL    : inp_b_muxed = inp_b ;
      BEQ     : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      BPOS    : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      BNEG    : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      BCEQ    : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      STAT    : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      POP     : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      PUSH    : inp_b_muxed = { inp_instruction.i.x, 16'd0 } ;
      FADDI   : inp_b_muxed = { inp_instruction.i.x, 16'd0 } ;
      FMULI   : inp_b_muxed = { inp_instruction.i.x, 16'd0 } ;
      ADDUI   : inp_b_muxed = { WORD_BITS { 1'bx } } ;
      default : inp_b_muxed = { WORD_BITS { 1'bx } } ;
    endcase
  end

  logic                                       sub;
  assign sub = (inp_instruction.r.op == FSUB);
  logic [WORD_BITS                      -1:0] add_res;
  fp32_add #(
    .ROUND(ROUND)
  ) ADDER (
    .clk(clk),
    .inp_a(inp_a),
    .inp_b(inp_b_muxed),
    .inp_op(sub),
    .out_res(add_res)
  );

  logic [WORD_BITS                      -1:0] mul_res;
  fp32_mul #(
    .ROUND(ROUND)
  ) MUL (
    .clk(clk),
    .inp_a(inp_a),
    .inp_b(inp_b_muxed),
    .out_res(mul_res)
  );

  opcode_t                                    inp_op_r;
  always_ff @(posedge clk) begin
    inp_op_r <= inp_instruction.r.op;
  end
  assign out_result = ( ( inp_op_r == FMUL ) | ( inp_op_r == FMULI ) ) ? mul_res : add_res;

endmodule