module object_buffer
  (
    // Outputs
    c0_obj_data, c1_obj_data, out_init_load, out_init_addr, out_init_data,
    // Inputs
    clk, in_init_load, in_init_addr, in_init_data, c0_obj_addr, c1_obj_addr, c0_obj_read, c1_obj_read
  );

  import types_pkg::*;

  // Control
  // input  logic                                reset_n;
  input  logic                                clk;

  // Input initialisation
  input  logic                                in_init_load;
  input  logic [OBJ_BUFFER_ADDR_BITS    -1:0] in_init_addr;
  input  logic [WORD_BITS               -1:0] in_init_data;

  // Output initialisation
  output logic                                out_init_load;
  output logic [OBJ_BUFFER_ADDR_BITS    -1:0] out_init_addr;
  output logic [WORD_BITS               -1:0] out_init_data;

  // Core 0 interface
  input  logic [OBJ_BUFFER_ADDR_BITS    -1:0] c0_obj_addr;
  input  logic                                c0_obj_read;
  output logic [WORD_BITS               -1:0] c0_obj_data;

  // Core 1 interface
  input  logic [OBJ_BUFFER_ADDR_BITS    -1:0] c1_obj_addr;
  input  logic                                c1_obj_read;
  output logic [WORD_BITS               -1:0] c1_obj_data;


  reg [WORD_BITS -1:0] objbuf [0:2**OBJ_BUFFER_ADDR_BITS -1];

  logic [OBJ_BUFFER_ADDR_BITS           -1:0] ram_addr_a;
  logic [OBJ_BUFFER_ADDR_BITS           -1:0] ram_addr_b;


  assign ram_addr_a = ( in_init_load ? in_init_addr : c0_obj_addr );
  assign ram_addr_b = c1_obj_addr;
  assign out_init_data = c0_obj_data;

  always_ff @(posedge clk) begin
    out_init_load <= in_init_load;
    if ( in_init_load ) begin
      out_init_addr <= in_init_addr;
    end
  end

  always_ff @(posedge clk) begin
    if ( in_init_load | c0_obj_read ) begin
      if ( in_init_load ) begin
        objbuf[ram_addr_a] <= in_init_data;
        c0_obj_data <= in_init_data;
      end
      else begin
        c0_obj_data <= objbuf[ram_addr_a];
      end
    end
    if( c1_obj_read ) begin
      c1_obj_data <= objbuf[ram_addr_b];
    end
  end

endmodule

