module register_file
  (
    // Outputs
    do_a, do_b,
    // Inputs
    clk, di_w, we, addr_a, addr_b, addr_w
  );

  import types_pkg::*;

  // Control
  //input  logic                                reset_n;
  input  logic                                clk;

  // Read A
  input  logic [REGISTER_ADDR_BITS      -1:0] addr_a;
  output logic [WORD_BITS               -1:0] do_a;

  // Read B
  input  logic [REGISTER_ADDR_BITS      -1:0] addr_b;
  output logic [WORD_BITS               -1:0] do_b;

  // Write
  input  logic                                we;
  input  logic [REGISTER_ADDR_BITS      -1:0] addr_w;
  input  logic [WORD_BITS               -1:0] di_w;


  
  // (* ram_style = "block" *) reg [WORD_BITS -1:0] ram [0:2**REGISTER_ADDR_BITS -1];
  reg [WORD_BITS -1:0] ram [0:2**REGISTER_ADDR_BITS -1];


  // Read-first
  // logic [WORD_BITS                      -1:0] r_a;
  // logic [WORD_BITS                      -1:0] r_b;
  // always @(posedge clk)
  // begin
  //   r_a <= ram[addr_a];
  //   r_b <= ram[addr_b];
  //   if (we)
  //     ram[addr_w] <= di_w;
  // end

  // logic                                       a_sel;
  // logic                                       b_sel;
  // always_ff @(posedge clk)
  // begin
  //   a_sel <= (addr_a[4:0] == 0);
  //   b_sel <= (addr_b[4:0] == 0);
  // end

  // assign do_a = ( a_sel ? 0 : r_a );
  // assign do_b = ( b_sel ? 0 : r_b );




  // Read-first
  always @(posedge clk)
  begin
    do_a <= (addr_a[4:0] == 0) ? 0 : ram[addr_a];
    do_b <= (addr_b[4:0] == 0) ? 0 : ram[addr_b];
    if (we)
      ram[addr_w] <= di_w;
  end


  // Read-first with better mapping to BRAM : )
  // (* ram_style = "block" *) reg [WORD_BITS -1:0] rama [0:2**REGISTER_ADDR_BITS -1];
  // logic                                       reset_a;
  // assign reset_a = addr_a[4:0] == 0;

  // // logic [WORD_BITS                      -1:0] r_a;
  // always @(posedge clk) begin
  //   if (reset_a) begin
  //     do_a <= 0;
  //   end
  //   else begin
  //     do_a <= rama[addr_a];
  //   end
  //   if (we) begin
  //     rama[addr_w] <= di_w;
  //   end
  // end

  // (* ram_style = "block" *) reg [WORD_BITS -1:0] ramb [0:2**REGISTER_ADDR_BITS -1];
  // logic                                       reset_b;
  // assign reset_b = addr_b[4:0] == 0;

  // always @(posedge clk) begin
  //   if (reset_b) begin
  //     do_b <= 0;
  //   end
  //   else begin
  //     do_b <= ramb[addr_b];
  //   end
  //   if (we) begin
  //     ramb[addr_w] <= di_w;
  //   end
  // end

endmodule