module reverse_regslice
  (
    // Outputs
    slave_ready, master_valid, master_data,
    // Inputs
    clk, reset_n, slave_valid, master_ready, slave_data
  );

  parameter DATA_WIDTH = 8;

  import types_pkg::*;

  // Control
  input  logic                                reset_n;
  input  logic                                clk;

  // Slave interface
  input  logic                                slave_valid;
  output logic                                slave_ready;
  input  logic [DATA_WIDTH              -1:0] slave_data;

  // Master interface
  output logic                                master_valid;
  input  logic                                master_ready;
  output logic [DATA_WIDTH              -1:0] master_data;

  // Internal signals
  logic [DATA_WIDTH                     -1:0] tmp_r;
  logic                                       tmp_r_wren;

  logic                                       tmp_free_nxt;
  logic                                       tmp_free_r;

  logic                                       slave_accept;
  logic                                       master_accept;

  assign slave_accept = slave_ready & slave_valid;
  assign master_accept = master_ready & master_valid;

  assign slave_ready = tmp_free_r;
  assign master_valid = slave_valid | !tmp_free_r;

  assign master_data = ( tmp_free_r ? slave_data : tmp_r );

  assign tmp_free_nxt = (master_accept | tmp_free_r) & !tmp_r_wren; // Unsure about this one!

  assign tmp_r_wren = slave_accept & !master_accept;

  always_ff @(posedge clk or negedge reset_n) begin
    if ( !reset_n ) begin
      tmp_free_r <= 1'b1;
    end
    else begin
      tmp_free_r <= tmp_free_nxt;
    end
  end

  always_ff @(posedge clk) begin
    if ( slave_accept ) begin
      tmp_r <= slave_data;
    end
  end

endmodule