package types_pkg;
  parameter WORD_BITS = 32;
  parameter BYTE_BITS = 8;
  parameter INSTRUCTION_BITS = 31;
  parameter PC_BITS = 8;
  parameter OBJ_BUFFER_ADDR_BITS = 7;

  parameter REGISTER_ADDR_BITS = 7;

  parameter SCREEN_COORD_BITS = 19;

  parameter REQUEST_BITS = 1 + SCREEN_COORD_BITS + 3*WORD_BITS;
  parameter RESULT_BITS = 1 + SCREEN_COORD_BITS + 3*BYTE_BITS;

  parameter THREADS = 4;

  typedef enum logic[1:0]
  {
    NONE       = 2'b00,
    START      = 2'b01,
    RESBUF_GNT = 2'b10
  } command_t;

  typedef enum logic[1:0]
  {
    IDLE       = 2'b00,
    BUSY       = 2'b01,
    RESBUF_REQ = 2'b10
  } status_t;

  typedef enum logic[4:0]
  {
    ADD   = 5'b00000,
    SUB   = 5'b00001,
    SRL   = 5'b00010,
    LUI   = 5'b00011,
    ADDI  = 5'b00100,
    SUBI  = 5'b00101,
    MULI  = 5'b00110,
    LW    = 5'b00111,
    LB    = 5'b01000,
    LFX   = 5'b01001,
    FADD  = 5'b01010,
    FSUB  = 5'b01011,
    FMUL  = 5'b01100,
    BEQ   = 5'b01101,
    BPOS  = 5'b01110,
    BNEG  = 5'b01111,
    BCEQ  = 5'b10000,
    STAT  = 5'b10001,
    POP   = 5'b10010,
    PUSH  = 5'b10011,
    FADDI = 5'b10100,
    FMULI = 5'b10101,
    ADDUI = 5'b10110
  } opcode_t;


  typedef struct packed
  {
    opcode_t      op;
    logic [4:0]   rd;
    logic [4:0]   ra;
    logic [4:0]   rb;
    logic [10:0]  unused;
  } r_type_struct_t;

  typedef struct packed
  {
    opcode_t      op;
    logic [4:0]   rd;
    logic [4:0]   ra;
    logic [15:0]  x;
  } i_type_struct_t;

  typedef struct packed
  {
    opcode_t      op;
    logic [4:0]   unused;
    logic [4:0]   ra;
    logic [4:0]   rb;
    command_t     y;
    logic         unused_2;
    logic [7:0]   x;
  } b_type_struct_t;

  typedef union packed
  {
    r_type_struct_t  r;
    i_type_struct_t  i;
    b_type_struct_t  b;
  } instruction_union_t;
  
endpackage : types_pkg