<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:template match="/">
    <html>
      <head/>
      <body>
        <h1>
          <xsl:value-of select="all_datastructures/title"/>
        </h1>
        <h2>Registers</h2>
        <xsl:apply-templates mode="reg_toc"/>
        <h2>Enums</h2>
        <xsl:apply-templates mode="enum_toc"/>
        <h2>Descriptors</h2>
        <xsl:apply-templates mode="descr_toc"/>
        <hr/>
        <xsl:apply-templates/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="descriptor" mode="descr_toc">
    <a href="#{type}"><xsl:value-of select="type"/></a><br/>
  </xsl:template>
  <xsl:template match="descriptor" mode="reg_toc"/>
  <xsl:template match="descriptor" mode="enum_toc"/>

  <xsl:template match="para">
    <p><xsl:apply-templates/></p>
  </xsl:template>

  <xsl:template match="link">
    <a href="#{.}"><xsl:value-of select="."/></a>
  </xsl:template>

  <xsl:template match="field">
    <tr bgcolor="#eeeeee">
      <xsl:apply-templates/>
    </tr>
  </xsl:template>
  <xsl:template match="word">
    <td style="padding: 5px;">
      <xsl:apply-templates/>
    </td>
  </xsl:template>
  <xsl:template match="bits">
    <td style="padding: 5px;">
      <xsl:apply-templates/>
    </td>
  </xsl:template>
  <xsl:template match="name">
    <td style="padding: 5px;">
      <xsl:apply-templates/>
    </td>
  </xsl:template>
  <xsl:template match="usage">
    <td style="padding: 5px;">
      <xsl:apply-templates/>
    </td>
  </xsl:template>

  <xsl:template match="description">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="descriptor">
    <h2><a name="{type}"><xsl:value-of select="type"/></a></h2>
    <xsl:apply-templates/>
  </xsl:template>


  <xsl:template match="descriptor_fields">
    <div>
      <table width="1000" border="1" bordercolor="#000000" style="border-collapse:collapse;">
        <tr bgcolor="#88bb88">
          <th style="padding: 5px; text-align:left">word</th>
          <th style="padding: 5px; text-align:left">bits</th>
          <th style="padding: 5px; text-align:left">name</th>
          <th style="padding: 5px; text-align:left">usage</th>
        </tr>
        <xsl:apply-templates/>
      </table>
    </div>
  </xsl:template>

  <xsl:template match="type"/>
  <xsl:template match="title"/>
  <xsl:template match="title" mode="reg_toc"/>
  <xsl:template match="title" mode="enum_toc"/>
  <xsl:template match="title" mode="descr_toc"/>


  <xsl:template match="register" mode="descr_toc"/>
  <xsl:template match="register" mode="enum_toc"/>
  <xsl:template match="register" mode="reg_toc">
    <a href="#{type}"><xsl:value-of select="type"/></a><br/>
  </xsl:template>
  <xsl:template match="register">
    <h2><a name="{type}"><xsl:value-of select="type"/></a></h2>
    <PRE>
      ADDRESS: <xsl:value-of select="address"/>
      ACCESS : <xsl:value-of select="access"/>
    </PRE>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="reset_val">
    <td style="padding: 5px;">
      <xsl:apply-templates/>
    </td>
  </xsl:template>

  <xsl:template match="register_fields">
    <div>
      <table width="1000" border="1" bordercolor="#000000" style="border-collapse:collapse;">
        <tr bgcolor="#bb8888">
          <th style="padding: 5px; text-align:left">bits</th>
          <th style="padding: 5px; text-align:left">name</th>
          <th style="padding: 5px; text-align:left">reset_val</th>
          <th style="padding: 5px; text-align:left">usage</th>
        </tr>
        <xsl:apply-templates/>
      </table>
    </div>
  </xsl:template>

  <xsl:template match="address"/>
  <xsl:template match="access"/>


  <xsl:template match="value">
    <tr bgcolor="#eeeeee">
      <xsl:apply-templates/>
    </tr>
  </xsl:template>

  <xsl:template match="encoding">
    <td style="padding: 5px;">
      <xsl:apply-templates/>
    </td>
  </xsl:template>

  <xsl:template match="symbol">
    <td style="padding: 5px;">
      <xsl:apply-templates/>
    </td>
  </xsl:template>

  <xsl:template match="enum" mode="reg_toc"/>
  <xsl:template match="enum" mode="descr_toc"/>
  <xsl:template match="enum" mode="enum_toc">
    <a href="#{type}"><xsl:value-of select="type"/></a><br/>
  </xsl:template>
  <xsl:template match="enum">
    <h2><a name="{type}"><xsl:value-of select="type"/></a></h2>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="value_list">
    <div>
      <table width="1000" border="1" bordercolor="#000000" style="border-collapse:collapse;">
        <tr bgcolor="#8888bb">
          <th style="padding: 5px; text-align:left">encoding</th>
          <th style="padding: 5px; text-align:left">symbol</th>
          <th style="padding: 5px; text-align:left">usage</th>
        </tr>
        <xsl:apply-templates/>
      </table>
    </div>
  </xsl:template>

</xsl:stylesheet>
  
 
