import struct
import numpy as np
import random


def as_float(a):
    return struct.unpack('<f', struct.pack('<l', np.int32(a)))[0]


def as_int(a):
    return np.uint32(struct.unpack('<l', struct.pack('<f', a))[0])


def fp32_mul(a, b):
    a_sgn = a >> 31
    a_exp = (a >> 23) & 0xff
    a_sfc_enc = a & 0x7fffff
    b_sgn = b >> 31
    b_exp = (b >> 23) & 0xff
    b_sfc_enc = b & 0x7fffff

    # Assuming normal numbers
    a_sfc = a_sfc_enc | 0x800000
    b_sfc = b_sfc_enc | 0x800000

    # print('a_sgn: {}, a_exp: {}, a_sfc: {}'.format(a_sgn, a_exp, a_sfc))
    # print('b_sgn: {}, b_exp: {}, b_sfc: {}'.format(b_sgn, b_exp, b_sfc))
    # print a_sfc*2.0**-(23 + 127 - a_exp)

    prd_sfc = a_sfc * b_sfc

    prd_exp = a_exp + b_exp - 127

    prd_sfc_rnd = (prd_sfc >> 22) & 1
    prd_sfc_sty = (prd_sfc & 0x3fffff) != 0

    # print('{:055b}'.format(prd_sfc))
    prd_sfc >>= 23

    if prd_sfc & (1 << 24):
        prd_sfc_sty |= prd_sfc_rnd
        prd_sfc_rnd = prd_sfc & 1

        prd_sfc >>= 1
        prd_exp += 1

        # print('{:32b}'.format(prd_sfc))

    prd_val = (prd_exp << 23) | (prd_sfc & 0x7fffff)

    prd_sfc_odd = prd_sfc & 1
    round_up = prd_sfc_rnd & (prd_sfc_sty | prd_sfc_odd)

    if round_up:
        prd_val += 1

    if prd_exp >= 255:
        prd_val = 0x7f800000

    if prd_exp <= 0:
        prd_val = 0

    if (a_exp == 0) or (b_exp == 0):
        prd_val = 0

    if a_sgn ^ b_sgn:
        prd_val |= 0x80000000

    return prd_val

if __name__ == '__main__':
    seed = random.randint(0, 2**64)
    # seed = 5027199829991635945
    # seed = 3331868671836831859
    # seed = 5413340962765161605
    # seed = 3540844680517972993  # NaN result
    # seed = 7877261434346259817  # Ovf
    # seed = 4302492382549411898  # NaN
    # seed = 10323801987756574392  # Subnormal input..
    seed = 14924297330718698096  # Subnormal input..
    print('Seed: {}'.format(seed))
    myRand = random.seed(seed)
    for a in np.linspace(-5, 5, 101, dtype=np.float32):
        for b in np.linspace(-5, 5, 101, dtype=np.float32):
            # print('-' * 60)
            # a = as_float(random.getrandbits(32))
            # b = as_float(random.getrandbits(32))
            # print(as_float(fp32_mul(as_int(a), as_int(b))))
            # print(np.float64(np.float32(a) * np.float32(b)))

            res_calc = np.uint32(fp32_mul(as_int(a), as_int(b)))
            with np.errstate(over='ignore'):  # No warn on overflow
                res_corr = as_int(np.float32(a) * np.float32(b))

            res_corr_exp = (res_corr >> 23) & 0xff
            # print('Calculated: {:032b}'.format(res_calc))
            # print('Correct   : {:032b}'.format(res_corr))

            a_int = as_int(a)
            b_int = as_int(b)

            a_denorm = (((a_int >> 23) & 0xff) == 0) and (a_int & 0x7fffff)
            b_denorm = (((b_int >> 23) & 0xff) == 0) and (b_int & 0x7fffff)
            corr_denorm = (((res_corr >> 23) & 0xff) == 0) and (res_corr & 0x7fffff)

            if a_denorm or b_denorm:
                continue

            if res_calc != res_corr:
                print('-' * 60)
                print('{} * {}'.format(a, b))
                print('Calculated: {:032b} | {}'.format(res_calc, as_float(res_calc)))
                print('Correct   : {:032b} | {}'.format(res_corr, as_float(res_corr)))          
                if not corr_denorm:  # If subnormal result - fts!
                    if res_corr_exp != 0xff:  # Don't care about NaNs....
                        print('Results differ!' + '#' * 60)
                elif (res_calc & 0x7fffffff) != 0:
                    print('Should have flushed to zero!' + '#' * 60)
