import struct
import numpy as np
import random


def as_float(a):
    return struct.unpack('<f', struct.pack('<l', np.int32(a)))[0]


def as_int(a):
    return np.uint32(struct.unpack('<l', struct.pack('<f', a))[0])


def fp32_sqrt(a):
    a_sgn = a >> 31
    a_exp = (a >> 23) & 0xff
    a_sfc_enc = a & 0x7fffff

    # Assuming normal numbers
    a_sfc = a_sfc_enc | 0x800000

    # print('a_sgn: {}, a_exp: {}, a_sfc: {}'.format(a_sgn, a_exp, a_sfc))
    # print a_sfc*2.0**-(23 + 127 - a_exp)

    res_sfc = int(((a_sfc>>(a_exp&1))**.5)*2**23)
    # print('                    {:025b} : {}'.format(res_sfc, res_sfc))

    lz = 0
    while (res_sfc & 1<<44) == 0:
        lz += 1
        res_sfc <<= 1
    # print lz
    # print bin((res_sfc >> 21)& 0x7fffff)

    # print('             {:08b}'.format(a_exp))


    ebxpbias = (a_exp+127)

    if ebxpbias & 1:
        res_exp = ebxpbias>>1
    else:
        res_exp = ebxpbias>>1

    res_exp = (a_exp>>1) + 63 + (a_exp&1)


    # print('             {:08b}'.format(res_exp))    

    return ((res_sfc >> 21)& 0x7fffff) | res_exp<<23

if __name__ == '__main__':
    seed = random.randint(0, 2**64)
    # seed = 5027199829991635945
    # seed = 3331868671836831859
    # seed = 5413340962765161605
    # seed = 3540844680517972993  # NaN result
    # seed = 7877261434346259817  # Ovf
    # seed = 4302492382549411898  # NaN
    # seed = 10323801987756574392  # Subnormal input..
    seed = 14924297330718698096  # Subnormal input..
    print('Seed: {}'.format(seed))
    myRand = random.seed(seed)
    for a in np.linspace(0, 1000, 10001, dtype=np.float32):
        # print('-' * 60)
        # a = as_float(random.getrandbits(31))

        res_calc = np.uint32(fp32_sqrt(as_int(a)))
        with np.errstate(over='ignore'):  # No warn on overflow
            res_corr = as_int(np.float32(a)**.5)

        res_corr_exp = (res_corr >> 23) & 0xff
        # print('Calculated: {:032b}'.format(res_calc))
        # print('Correct   : {:032b}'.format(res_corr))

        a_int = as_int(a)

        a_denorm = (((a_int >> 23) & 0xff) == 0) and (a_int & 0x7fffff)
        corr_denorm = (((res_corr >> 23) & 0xff) == 0) and (res_corr & 0x7fffff)

        # if a_denorm:
        #     continue

        # if res_calc != res_corr:
        if res_calc not in [res_corr, res_corr+1, res_corr-1]:
            print('-' * 60)
            print('{}'.format(a))
            print('Calculated: {:032b} | {}'.format(res_calc, as_float(res_calc)))
            print('Correct   : {:032b} | {}'.format(res_corr, as_float(res_corr)))          
            if not corr_denorm:  # If subnormal result - fts!
                if res_corr_exp != 0xff:  # Don't care about NaNs....
                    print('Results differ!' + '#' * 60)
            elif (res_calc & 0x7fffffff) != 0:
                print('Should have flushed to zero!' + '#' * 60)
