import subprocess

program_file = '../isasim/rayman/program.txt'
#program_file = 'assembler_test.txt'

types_pkg = '../verilog/types_pkg.sv'

regmap = ['0']

regmap = ['0',
          'dir_x',
          'dir_y',
          'dir_z',
          'orig_x',
          'orig_y',
          'orig_z',
          'color_r',
          'color_g',
          'color_b',
          'k_refl',
          'rec_surf',
          'isect_surface',
          'isect_normal_x',
          'isect_normal_y',
          'isect_normal_z',
          'rec_left'] + [str(a) for a in range(17, 32)]

#regmap = [str(a) for a in range(32)]

with open(types_pkg) as fh:
    lines = fh.readlines()

    for i, line in enumerate(lines):
        if '{' in line:
            start = i
        if '}' in line and 'opcode_t' in line:
            stop = i
            break

opcodes = ''.join(lines[start + 1:stop])

opcodes = opcodes.split(',')
opcodes = [[b.strip().lower() for b in a.split("= 5'b")] for a in opcodes]

opcodes = dict(opcodes)


def d2b(d, n):
    if type(d) == str:
        tmp = d.replace('$', '').strip()
        tmp = int(tmp, 0)
    else:
        tmp = d
    formatted = '{:0{}b}'.format(tmp, n)
    if len(formatted) != n:
        print('Could not format {} into {} bits. Got {} ({} bits)'.format(
            d, n, formatted, len(formatted)))
    return formatted


def reg2b(reg):
    reg = str(reg)
    reg = reg.strip()
    reg = reg.replace('$', '')
    try:
        i = regmap.index(reg)
    except ValueError, e:
        i = len(regmap)
        regmap.append(reg)
    return d2b(i, 5)


def r_type(op, rd, ra, rb):
    tmp = opcodes[op.lower().strip()] + '_'
    tmp += reg2b(rd) + '_'
    tmp += reg2b(ra) + '_'
    tmp += reg2b(rb) + '_'
    tmp += '0' * 11
    return tmp


def i_type(op, rd, ra, x):
    tmp = opcodes[op.lower().strip()] + '_'
    tmp += reg2b(rd) + '_'
    tmp += reg2b(ra) + '_'
    tmp += d2b(x, 16)
    return tmp


def b_type(op, ra, rb, y, x):
    tmp = opcodes[op.lower().strip()] + '_'
    tmp += '00000_'
    tmp += reg2b(ra) + '_'
    tmp += reg2b(rb) + '_'
    tmp += d2b(y, 2) + '_'
    tmp += '0_'
    tmp += d2b(x, 8)
    return tmp


with open(program_file) as fh:
    lines = []
    labels = {}
    i = 0
    for line in fh.readlines():
        line = line.split(';')[0].strip()
        if line:
            if ':' in line:
                labels[line.split(':')[0]] = i
            else:
                lines.append(line)
                i += 1
    inv_labels = {v: k for k, v in labels.items()}
    try:
        label_length = max([len(a) for a in labels])
    except:
        label_length = 1

result = ''

for i, line in enumerate(lines):
    s = line.replace(',', '').split()
    op = s[0].lower()
    s[0] = op

    if op in ['lw', 'lb', 'lfx']:
        loc = s[2].split('[$')
        imm = loc[0]
        reg = loc[1][:-1]
        tmpresult = i_type(op, s[1], reg, imm)

    elif op == 'beq':
        tmpresult = b_type(op, s[1], s[2], 0, labels[s[3]])

    elif op == 'goto':
        tmpresult = b_type('beq', 0, 0, 0, labels[s[1]])

    elif op in ['bpos', 'bneg']:
        tmpresult = b_type(op, s[1], 0, 0, labels[s[2]])

    elif op == 'bceq':
        tmpresult = b_type(op, 0, 0, s[1], labels[s[2]])

    elif op in ['muli', 'addi', 'addui', 'subi', 'faddi', 'fmuli']:
        tmpresult = i_type(*s)

    elif op in ['add', 'sub', 'fadd', 'fsub', 'fmul']:
        tmpresult = r_type(*s)

    elif op == 'srl':
        tmpresult = r_type(op, s[1], s[2], 0)

    elif op == 'lui':
        tmpresult = i_type(op, s[1], 0, s[2])

    elif op == 'stat':
        tmpresult = i_type(op, 0, 0, s[1])

    elif op == 'pop':
        tmpresult = r_type(op, s[1], 0, 0)

    elif op == 'push':
        tmpresult = i_type(op, 0, s[1], 0x3f80)

    else:
        tmpresult = 'NOT IMPLEMENTED?'

    label = inv_labels[i] if i in inv_labels else ''
    result += '{:37} // {:{}}: {}\n'.format(tmpresult,
                                            label, label_length, line)

print result

print('Final register mapping:')
for i, mnemonic in enumerate(regmap):
    print('{:2d} ({:05b}): ${}'.format(i, i, mnemonic))

with open('../verilog/instruction_memory.dat', 'w') as fh:
    fh.write(result)


# cmd = ["rsync", "-rth",
#        "--exclude", "'.git*'",
#        "--exclude", "'*.pickle'",
#        "/Users/jonas/src/rayman/verilog/",
#        "jonasae@venus.tele.ntnu.no:/home/jonasae/src/rayman/verilog/"]

# subprocess.call(cmd)
