import struct
import numpy as np
import random

signtruthtable = set()


def as_float(a):
    return struct.unpack('<f', struct.pack('<l', np.int32(a)))[0]


def as_int(a):
    return np.uint32(np.uint32(struct.unpack('<l', struct.pack('<f', a))[0]))


def fp32_add(a, b, op=1):
    a = as_int(a)
    b = as_int(b)
    a_sgn = a >> 31
    a_exp = (a >> 23) & 0xff
    a_sfc_enc = a & 0x7fffff
    b_sgn = b >> 31
    b_exp = (b >> 23) & 0xff
    b_sfc_enc = b & 0x7fffff

    # Assuming normal numbers
    a_sfc = a_sfc_enc | 0x800000 if a_exp else a_sfc_enc
    b_sfc = b_sfc_enc | 0x800000 if b_exp else b_sfc_enc

    # print('a_sgn: {}, a_exp: {}, a_sfc: {} | {}'.format(
    #     a_sgn, a_exp, a_sfc, as_float(a)))
    # print('b_sgn: {}, b_exp: {}, b_sfc: {} | {}'.format(
    #     b_sgn, b_exp, b_sfc, as_float(b)))
    # print a_sfc*2.0**-(23 + 127 - a_exp)

    d = a_exp - b_exp
    res_exp = a_exp if d >= 0 else b_exp
    x_sfc = a_sfc if d >= 0 else b_sfc
    y_sfc = b_sfc if d >= 0 else a_sfc

    guard_bit = 0
    round_bit = 0
    sticky_bit = 0
    for i in range(abs(d)):
        sticky_bit |= round_bit
        round_bit = guard_bit
        guard_bit = y_sfc & 0b1
        y_sfc >>= 1

    # print('a_exp  : {:08b}, a_sfc  : {:024b}'.format(a_exp, a_sfc))
    # print('b_exp  : {:08b}, b_sfc  : {:024b}'.format(b_exp, b_sfc))
    # print('-' * 10)
    # print('res_exp: {:08b}, x_sfc  : {:024b}'.format(res_exp, x_sfc))
    # print('res_exp: {:08b}, y_sfc  : {:024b}, {} {} {}'.format(res_exp, y_sfc, guard_bit, round_bit, sticky_bit))
    # print('d: {}'.format(d))

    sgns = a_sgn ^ b_sgn
    eop = (op ^ sgns)

    x_sfc = (x_sfc << 3)
    y_sfc = (y_sfc << 3) | (guard_bit << 2) | (
        round_bit << 1) | (sticky_bit << 0)

    if (eop == 1):
        if (d == 0):
            if a_sfc_enc < b_sfc_enc:
                x_sfc = ~x_sfc
            else:
                y_sfc = ~y_sfc
        else:
            y_sfc = ~y_sfc

    res_sum = x_sfc + y_sfc + eop

    guard_bit = (res_sum >> 2) & 0b1
    round_bit = (res_sum >> 1) & 0b1
    sticky_bit = (res_sum >> 0) & 0b1

    res_sum >>= 3
    # print('{} - {} = {}'.format(x_sfc, y_sfc, res_sum))

    if (eop == 0) and (res_sum & (1 << 24)):
        sticky_bit |= round_bit
        round_bit = guard_bit
        guard_bit = res_sum & 0b1
        res_sum >>= 1
        res_exp += 1

    if (eop == 1) and (res_sum != 0):
        # print('a: 32\'h{:08x}, b: 32\'h{:08x}'.format(a, b))
        # print('{:024b}'.format(res_sum))
        while ((res_sum & (1 << 23)) == 0):
            res_sum = (res_sum << 1) | guard_bit
            guard_bit = round_bit
            round_bit = sticky_bit
            sticky_bit = 0
            res_exp -= 1

    rnd = guard_bit & ((res_sum & 0b1) | round_bit | sticky_bit)

    ##### ROUNDING "DISABLED"
    # res_sum += rnd

    if res_sum & (1 << 24):
        res_sum >>= 1
        res_exp += 1

    if (res_exp <= 0) or (res_sum == 0):
        res_sum = 0
        res_exp = 0

    # print('res_exp: {:08b}, res_sum: {:024b}'.format(res_exp, res_sum))

    # print('             {:08b}{:023b}'.format(res_exp, res_sum & 0x7fffff))

    sgn = d < 0

    # correct_res = as_int(as_float(a) - as_float(b)
    #                      if op else as_float(a) + as_float(b))
    # correct_sign = correct_res >> 31  # if correct_res != 0.0 else 1 # Hack
    # tmp = (int(sgn), int(d == 0), a_sgn, b_sgn, eop, int(
    #     a_sfc_enc < b_sfc_enc), int(a_sfc_enc == b_sfc_enc), correct_sign)
    # signtruthtable.add(tmp)

    signs = {'0 0 0 0 0 0 0': 0,
             '0 0 0 0 0 0 1': 0,
             '0 0 0 0 0 1 0': 0,
             '0 0 0 0 1 0 0': 0,
             '0 0 0 0 1 0 1': 0,
             '0 0 0 0 1 1 0': 0,
             '0 0 0 1 0 0 0': 0,
             '0 0 0 1 0 0 1': 0,
             '0 0 0 1 0 1 0': 0,
             '0 0 0 1 1 0 0': 0,
             '0 0 0 1 1 0 1': 0,
             '0 0 0 1 1 1 0': 0,
             '0 0 1 0 0 0 0': 1,
             '0 0 1 0 0 0 1': 1,
             '0 0 1 0 0 1 0': 1,
             '0 0 1 0 1 0 0': 1,
             '0 0 1 0 1 0 1': 1,
             '0 0 1 0 1 1 0': 1,
             '0 0 1 1 0 0 0': 1,
             '0 0 1 1 0 0 1': 1,
             '0 0 1 1 0 1 0': 1,
             '0 0 1 1 1 0 0': 1,
             '0 0 1 1 1 0 1': 1,
             '0 0 1 1 1 1 0': 1,
             '0 1 0 0 0 0 0': 0,
             '0 1 0 0 0 0 1': 0,
             '0 1 0 0 0 1 0': 0,
             '0 1 0 0 1 0 0': 0,
             '0 1 0 0 1 0 1': 0,
             '0 1 0 0 1 1 0': 1,
             '0 1 0 1 0 0 0': 0,
             '0 1 0 1 0 0 1': 0,
             '0 1 0 1 0 1 0': 0,
             '0 1 0 1 1 0 0': 0,
             '0 1 0 1 1 0 1': 0,
             '0 1 0 1 1 1 0': 1,
             '0 1 1 0 0 0 0': 1,
             '0 1 1 0 0 0 1': 1,
             '0 1 1 0 0 1 0': 1,
             '0 1 1 0 1 0 0': 1,
             '0 1 1 0 1 0 1': 0,
             '0 1 1 0 1 1 0': 0,
             '0 1 1 1 0 0 0': 1,
             '0 1 1 1 0 0 1': 1,
             '0 1 1 1 0 1 0': 1,
             '0 1 1 1 1 0 0': 1,
             '0 1 1 1 1 0 1': 0,
             '0 1 1 1 1 1 0': 0,
             '1 0 0 0 0 0 0': 0,
             '1 0 0 0 0 0 1': 0,
             '1 0 0 0 0 1 0': 0,
             '1 0 0 0 1 0 0': 1,
             '1 0 0 0 1 0 1': 1,
             '1 0 0 0 1 1 0': 1,
             '1 0 0 1 0 0 0': 0,
             '1 0 0 1 0 0 1': 0,
             '1 0 0 1 0 1 0': 0,
             '1 0 0 1 1 0 0': 1,
             '1 0 0 1 1 0 1': 1,
             '1 0 0 1 1 1 0': 1,
             '1 0 1 0 0 0 0': 1,
             '1 0 1 0 0 0 1': 1,
             '1 0 1 0 0 1 0': 1,
             '1 0 1 0 1 0 0': 0,
             '1 0 1 0 1 0 1': 0,
             '1 0 1 0 1 1 0': 0,
             '1 0 1 1 0 0 0': 1,
             '1 0 1 1 0 0 1': 1,
             '1 0 1 1 0 1 0': 1,
             '1 0 1 1 1 0 0': 0,
             '1 0 1 1 1 0 1': 0,
             '1 0 1 1 1 1 0': 0}
    tmp = (sgn, (d == 0), a_sgn, b_sgn, eop,
           (a_sfc_enc < b_sfc_enc), (res_sum == 0))
    tmp = ' '.join([str(int(x)) for x in tmp])
    res_sgn = signs[tmp]

    res_concat = res_sgn << 31
    # res_concat = correct_sign << 31
    res_concat |= (res_exp & 0xff) << 23
    res_concat |= (res_sum & 0x7fffff)

    return as_float(res_concat)
