import struct
import numpy as np
import random


def as_float(a):
    return struct.unpack('<f', struct.pack('<l', np.int32(a)))[0]


def as_int(a):
    return np.uint32(struct.unpack('<l', struct.pack('<f', a))[0])


def fp32_mul(a, b):
    a = as_int(a)
    b = as_int(b)
    a_sgn = a >> 31
    a_exp = (a >> 23) & 0xff
    a_sfc_enc = a & 0x7fffff
    b_sgn = b >> 31
    b_exp = (b >> 23) & 0xff
    b_sfc_enc = b & 0x7fffff

    # Assuming normal numbers
    a_sfc = a_sfc_enc | 0x800000
    b_sfc = b_sfc_enc | 0x800000

    # print('a_sgn: {}, a_exp: {}, a_sfc: {}'.format(a_sgn, a_exp, a_sfc))
    # print('b_sgn: {}, b_exp: {}, b_sfc: {}'.format(b_sgn, b_exp, b_sfc))
    # print a_sfc*2.0**-(23 + 127 - a_exp)

    prd_sfc = a_sfc * b_sfc

    prd_exp = a_exp + b_exp - 127

    prd_sfc_round = (prd_sfc >> 22) & 1
    prd_sfc_sticky = (prd_sfc & 0x3fffff) != 0

    # print('{:055b}'.format(prd_sfc))
    prd_sfc >>= 23

    if prd_sfc & (1 << 24):
        prd_sfc_sticky |= prd_sfc_round
        prd_sfc_round = prd_sfc & 1

        prd_sfc >>= 1
        prd_exp += 1

        # print('{:32b}'.format(prd_sfc))

    prd_val = (prd_exp << 23) | (prd_sfc & 0x7fffff)

    prd_sfc_odd = prd_sfc & 1
    round_up = prd_sfc_round & (prd_sfc_sticky | prd_sfc_odd)

    ##### ROUNDING "DISABLED"
    # if round_up:
    #     prd_val += 1

    if prd_exp >= 255:
        prd_val = 0x7f800000

    if prd_exp <= 0:
        prd_val = 0

    if (a_exp == 0) or (b_exp == 0):
        prd_val = 0

    if a_sgn ^ b_sgn:
        prd_val |= 0x80000000

    return as_float(prd_val)
