# ------------------------------------------------------------------------------
# Author       : Oystein Gjermundnes
# Organization : Norwegian University of Science and Technology (NTNU)
#                http://www.iet.ntnu.no/en
# Course       : Design of digital systems 1 (DDS1)
# Year         : 2015
# Fork         : git clone git@bitbucket.org:oysteingjermundnes/dds1.git
# Description  : Raytracer model written in python
# License      : This is free and unencumbered software released into the 
#                public domain (UNLICENSE)
# ------------------------------------------------------------------------------
import numpy as np

################################################################################
## Class for storing ARGB colors
#
# Class used for storing ARGB colors
#
################################################################################
class Color:
  ##############################################################################
  ## Constructor for the Color class
  #
  # @param r Red   component.
  # @param g Green component.
  # @param b Blue  component.
  ##############################################################################
  def __init__(self, r, g, b): 
    ## Red component
    self.r     = np.float32(r)
    ## Green component 
    self.g     = np.float32(g)   
    ## Blue component
    self.b     = np.float32(b)  

  ##############################################################################
  ## Print the color
  #
  # @param self The object pointer.
  ##############################################################################
  def __str__(self):
    return "RGB = (%f, %f, %f)" % (self.r, self.g, self.b)

  ##############################################################################
  ## Add two colors component by component
  #
  # @param self   The object pointer.
  # @param other  Pointer to the Color that is added to this Color.
  ##############################################################################
  def add(self, other):
    return Color(self.r + other.r,
                 self.g + other.g,
                 self.b + other.b)

  ##############################################################################
  ## Multiply the color with a scalar
  # 
  # @param self The object pointer.
  # @param k    Scalar value that is multiplied with each Color component.  
  ##############################################################################
  def scalar_mult(self, k):
    k = np.float32(k)
    return Color(self.r * k,
                 self.g * k,
                 self.b * k)

  ##############################################################################
  ## Clamp colors
  #
  # The color components are first clamped to the range 0.0 - 1.0 and then 
  # scaled by 255.
  #
  # @param self The object pointer.
  ##############################################################################
  def clamp(self):
    r = int(min(self.r, 1)*255)
    g = int(min(self.g, 1)*255)
    b = int(min(self.b, 1)*255)

    # Return the clamped color
    return (r,g,b)
