# ------------------------------------------------------------------------------
# Author       : Oystein Gjermundnes
# Organization : Norwegian University of Science and Technology (NTNU)
#                http://www.iet.ntnu.no/en
# Course       : Design of digital systems 1 (DDS1)
# Year         : 2015
# Fork         : git clone git@bitbucket.org:oysteingjermundnes/dds1.git
# Description  : Raytracer model written in python
# License      : This is free and unencumbered software released into the 
#                public domain (UNLICENSE)
# ------------------------------------------------------------------------------

from rayman.color import *

################################################################################
## Class used for capturing Material properties
#
# Class used for capturing material properties such as:
#   - Transparency
#   - Shininess
#   - Ambient color.Color
#   - Diffuse color.Color
#   - Specilar color.Color
#   - Reflection coefficient
#   - Refraction coefficient
#   - Color function 
################################################################################
class Material:
  ##############################################################################
  ## Constructor
  # @param self       Pointer to the Material object.
  # @param alpha      Transparency level (0.0: Fully transparent).
  # @param shininess  Specular factor.
  # @param ac         Ambient Color.
  # @param dc         Diffuse Color.
  # @param sc         Specular Color.
  # @param k_refl     Reflection coefficient.
  # @param k_refr     Refraction coefficient.
  # @param color_func Color function.
  ##############################################################################
  def __init__(self,
               shininess,
               ac,
               dc,
               sc,
               k_refl):
    ## Specular factor
    self.shininess    = shininess 
    ## Ambient color.Color
    self.ac           = ac        
    ## Diffuse color.Color
    self.dc           = dc        
    ## Specular color.Color
    self.sc           = sc        
    ## Reflection coefficient
    self.k_refl       = k_refl
