# ------------------------------------------------------------------------------
# Author       : Oystein Gjermundnes
# Organization : Norwegian University of Science and Technology (NTNU)
#                http://www.iet.ntnu.no/en
# Course       : Design of digital systems 1 (DDS1)
# Year         : 2015
# Fork         : git clone git@bitbucket.org:oysteingjermundnes/dds1.git
# Description  : Raytracer model written in python
# License      : This is free and unencumbered software released into the
#                public domain (UNLICENSE)
# ------------------------------------------------------------------------------

from rayman.vector import *
from rayman.ray import *
from rayman.sphere import *
import numpy as np

from rayman.memory import *


class Objbuffer():
    ##########################################################################
    # Constructor
    #
    # @param self      Pointer to the scene.
    # @param filename  Name of the file containing the scene description.
    ##########################################################################

    def __init__(self, ram):
        self.buffer = Memory()
        self.ram = ram

    def read_req(self, addr):
        # This must be the ugliest stuff ever written.
        for i in range(6):
            self.buffer.store_word(self.ram.load_word(addr + i*4), i*4)
        
        sphere_size = self.ram.load_word(addr + 12*4)
        sphere_ptr = self.ram.load_word(addr + 11*4)
        material_size = self.ram.load_word(addr + 14*4)
        material_ptr = self.ram.load_word(addr + 13*4)

        self.buffer.store_word(sphere_size, 6*4)

        for i in range(sphere_size * 6):
            int_addr = (7+i)*4
            self.buffer.store_word(self.ram.load_word(sphere_ptr + i*4), int_addr)

        int_addr += 4

        for i in range(material_size * 4):
            self.buffer.store_word(self.ram.load_word(material_ptr + i*4), int_addr + i*4)

        return 'DONE'