# ------------------------------------------------------------------------------
# Author       : Oystein Gjermundnes
# Organization : Norwegian University of Science and Technology (NTNU)
#                http://www.iet.ntnu.no/en
# Course       : Design of digital systems 1 (DDS1)
# Year         : 2015
# Fork         : git clone git@bitbucket.org:oysteingjermundnes/dds1.git
# Description  : Raytracer model written in python
# License      : This is free and unencumbered software released into the 
#                public domain (UNLICENSE)
# ------------------------------------------------------------------------------

from rayman.vector import *
from rayman.color import *
from rayman.isectpoint import *
################################################################################
## Class representing a Ray of light.
################################################################################
class Ray:

  ##############################################################################
  ## Constructor
  #
  # @param self Pointer to the ray.
  # @param o    vector.Vector representing the origin of the Ray.
  # @param d    vector.Vector representing the direction of the Ray.
  # @param t    String indicating the type of the ray {Shadow, Reflection, Refraction, Initial}
  ##############################################################################
  def __init__(self,o, d, t):
    ## Origin of the ray represented by a vector.Vector
    self.origin    = o
    ## Direction of the ray represented by a vector.Vector
    self.direction = d
    ## Pointer to the isectpoint.IsectPoint between the ray and a primitive.
    self.isect     = IsectPoint()
    ## Type of the ray {Shadow, Reflection, Refraction, Initial}
    self.type      = t

  ##############################################################################
  ## Print out key information about the Ray.
  #
  # The method prints Type, Origin, Direction, Intersection position and Color.
  #
  # @param self Pointer to the Ray.
  ##############################################################################
  def print(self):
    print("Ray:")
    print("  Type              : %s" % self.type)
    print("  Origin            : %s" % self.origin.str())
    print("  Direction         : %s" % self.direction.str())
    print("  Intersection pos  : %s" % self.isect.coord.str())
