# ------------------------------------------------------------------------------
# Author       : Oystein Gjermundnes
# Organization : Norwegian University of Science and Technology (NTNU)
#                http://www.iet.ntnu.no/en
# Course       : Design of digital systems 1 (DDS1)
# Year         : 2015
# Fork         : git clone git@bitbucket.org:oysteingjermundnes/dds1.git
# Description  : Raytracer model written in python
# License      : This is free and unencumbered software released into the 
#                public domain (UNLICENSE)
# ------------------------------------------------------------------------------

from rayman.color import *
from rayman.vector import *
from rayman.material import *
from rayman.ray import *

################################################################################ 
## Class representing Sphere primitive.Primitive. 
#
# This class contains a representation of Spheres. The class implements 
# functions for calculating isectpoint.IsectPoint with ray.Ray and the distance
# from the origin of the ray.Ray and to the isectpoint.IsectPoint with the Sphere.
################################################################################ 
class Sphere():

  ##############################################################################
  ## Constructor
  #
  # @param pos         Position of the Sphere represented as vector.Vector.
  # @param radius      Radius of the Sphere.
  # @param material_id The ID of the material.Material for this Sphere.
  ##############################################################################
  def __init__(self, pos, radius, material_id):
    ## ID of the material to use for this Primitive.
    self.material_id = material_id
    ## vector.Vector representing the center of the Sphere
    self.center = pos
    ## Radius of the Sphere.
    self.radius_squared = radius * radius
    self.radius_inverse = np.float32(1)/radius
