import os
import sys
import matplotlib
matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
import matplotlib.image as mpimg
import subprocess


try:
    import cPickle as pickle
except ImportError:
    import pickle
import numpy as np

pickles = [a for a in os.listdir('.') if '.pickle' in a and os.path.isfile(a)]
pickles.sort(reverse=True)

for i, p in enumerate(pickles):
    print('{:2}: {}'.format(i, p[:-7]))

if len(pickles) > 1:
    print('Choose statfile:')
    try:
        i = input()
        i = int(i)
    except:
        i = 0
else:
    i = 0


imagefilename = pickles[i][:-7] + '.png'
image = mpimg.imread(imagefilename)

fig = plt.figure()
axarr = []
axarr.append(plt.subplot2grid((3, 3), (0, 0), colspan=2, rowspan=2))
axarr.append(plt.subplot2grid((3, 3), (2, 0), colspan=3))
axarr.append(plt.subplot2grid((3, 3), (0, 2), rowspan=2))

imageplot = axarr[0].imshow(image, interpolation='nearest')

with open(pickles[i], 'rb') as fh:
    stats = pickle.load(fh)


total_cycles = sum([a['cycles'] for a in stats])
max_cycles = max([a['cycles'] for a in stats])
width = max([a['position'][0] for a in stats]) + 1
height = max([a['position'][1] for a in stats]) + 1

num_subi = sum([a['instructions']['subi'] for a in stats])
print('Total subi: {:,}'.format(num_subi))

operations = set()
maxops = 0
for a in stats:
    operations.update(a['instructions'].keys())
    maxops = max(maxops, max(a['instructions'].values()))

operations = list(operations)
operations.sort()

print('(w, h): ({}, {})'.format(width, height))
print('Total cycles: {:,}'.format(total_cycles))
print('Average cyles per pixel: {:.2f}'.format(
    float(total_cycles) / (width * height)))


def onpick(event):
    if event.button != 1:
        return
    if event.inaxes not in (axarr[0], ax2):
        return
    x = min(int(round(event.xdata)), width - 1)
    y = min(int(round(event.ydata)), height - 1)
    i = x + y * width
    assert (x, y) == stats[i]['position']

    s = stats[i]
    axarr[1].cla()
    axarr[1].set_title('(x,y): ({}, {})'.format(*s['position']))
    axarr[1].set_xlabel('Cycle')
    axarr[1].set_ylabel('Instruction')
    axarr[1].plot(s['history'][:-1])
    axarr[1].set_xlim([0, max_cycles])
    axarr[1].set_ylim([0, s['history'][-1]])

    D = s['instructions']
    num_ops = [D[x] if x in D else 0 for x in operations]

    axarr[2].cla()
    axarr[2].bar(range(len(operations)), num_ops, align='center')
    axarr[2].set_xticks(range(len(operations)))
    axarr[2].set_xticklabels(operations, rotation=45, ha='right')
    axarr[2].set_xlim([0 - .5, len(operations) - .5])
    axarr[2].set_ylim([0, maxops])

    fig.canvas.draw()


fig.canvas.mpl_connect('button_press_event', onpick)
fig.canvas.mpl_connect('motion_notify_event', onpick)
figManager = plt.get_current_fig_manager()
# figManager.window.state('zoomed')
figManager.resize(*figManager.window.maxsize())

# # plt.hist([a['cycles'] for a in stats], 50, normed=0)
# # plt.xlabel('Instructions')
# # plt.ylabel('Pixels')
# # plt.show()


fig2 = plt.figure()
ax2 = plt.subplot()
data = np.array([a['cycles'] for a in stats], dtype=np.float)
data = data.reshape([height, width])

cmap = plt.cm.viridis
norm = plt.Normalize(vmin=data.min(), vmax=data.max())

# map the normalized data to colors
# image is now RGBA (512x512x4) 
image = cmap(norm(data))

# save the image
plt.imsave('colormapped.png', image)
print('min:{}, max:{}'.format(data.min(), data.max()))

implt = ax2.imshow(data, interpolation='nearest')
fig2.colorbar(implt)

fig2.canvas.mpl_connect('button_press_event', onpick)
fig2.canvas.mpl_connect('motion_notify_event', onpick)


plt.show()
