import os
import subprocess
import matplotlib
import matplotlib.pyplot as plt
from matplotlib import style
import pandas as pd
matplotlib.rcParams['pdf.fonttype'] = 42

FMAX = 100e6
THREADS = 4
MEANCYCLES = 10000

cmd = ["rsync", "-rth",
       "jonasae@venus.tele.ntnu.no:/home/jonasae/src/rayman/verilog/results/",
       "/Users/jonas/src/rayman/verilog/results/"]

subprocess.call(cmd)


files = [a for a in os.listdir(cmd[-1]) if '.txt' in a]
files.sort(reverse=True)

for i, p in enumerate(files):
    print('{:2}: {}'.format(i, p[:-4]))

if len(files) > 1:
    print('Choose statfile:')
    try:
        i = input()
        i = int(i)
    except:
        i = 0
else:
    i = 0

logfile = os.path.join(cmd[-1], files[i])

t = []
outstanding_requests = []
active_threads = []
with open(logfile) as fh:
    num_dcores, reqcycle = [int(a) for a in fh.readline().split()]
    for line in fh.readlines()[1:]:
        x, y, z = [int(a) for a in line.split()]
        t.append(x / 2)
        outstanding_requests.append(y)
        active_threads.append(z)

titlefmt = 'Dcores: {}, Reqcycle: {}\nCycles: {:,}, FPS@{:.2f}MHz: {:.2f}'
title = titlefmt.format(
    num_dcores, reqcycle, t[-1], float(FMAX) / 1e6, float(FMAX) / t[-1])
print(title)

fig = plt.figure()
style.use('ggplot')


ax = plt.gca()
titlefmt = 'Dcores: {}, Reqcycle: {}\nCycles: {:,}, FPS@{:.2f}MHz: {:.2f}'
ax.set_title(title)
ax.set_xlabel('Cycle')
# ax.plot(t, outstanding_requests, label='Outstanding requests')
ax.plot(t, active_threads, label='Active threads')

all_active = pd.Series([0] + active_threads, index=[0] + t)
all_active = all_active.reindex(range(0, t[-1] + 1), method='pad')
mean_active = all_active.rolling(window=MEANCYCLES, center=True).mean()
ax.plot(mean_active,  # .reindex(range(0, t[-1] + 1, MEANCYCLES)),
        label='Active threads (rolling mean, N={:,})'.format(MEANCYCLES))

ax.plot([0, t[-1]], [num_dcores * 2 * THREADS]
        * 2, label='Available threads')
ax.set_xlim([0, t[-1]])
ax.set_ylim([0, max(max(outstanding_requests), max(active_threads))])
ax.get_xaxis().set_major_formatter(plt.FuncFormatter(lambda x, loc: "{:,}".format(int(x))))

plt.legend(loc=8)

# plt.show()
plt.savefig('active_threads.png', bbox_inches='tight', dpi=600)
plt.savefig('active_threads.pdf', bbox_inches='tight')
