module drain_property
  (
    // Outputs
    slave_ready, master_valid, master_data, core_valid, core_data,
    // Inputs
    clk, reset_n, slave_valid, master_ready, slave_data, core_ready
  );

  import types_pkg::*;

  // Control
  input  logic                                reset_n;
  input  logic                                clk;

  // Slave interface
  input  logic                                slave_valid;
  input  logic                                slave_ready;
  input  logic [RESULT_BITS             -1:0] slave_data;

  // Master interface
  input  logic                                master_valid;
  input  logic                                master_ready;
  input  logic [RESULT_BITS             -1:0] master_data;

  // Core interface
  input  logic                                core_valid;
  input  logic                                core_ready;
  input  logic [RESULT_BITS             -1:0] core_data;


  logic                                       slave_accept;
  logic                                       master_accept;
  logic                                       core_accept;
  assign slave_accept = slave_ready & slave_valid;
  assign master_accept = master_ready & master_valid;
  assign core_accept = core_ready & core_valid;

  property master_data_stable;
    @(posedge clk) disable iff(!reset_n) (master_valid & !master_ready) |=> $stable(master_data);
  endproperty
  a_master_data_stable: assert property (master_data_stable) else $display($time," ns",,,"master_data_stable FAIL");

  property slave_accept_to_master_valid;
    @(posedge clk) disable iff(!reset_n) (slave_accept|core_accept) |=> master_valid;
  endproperty
  a_slave_accept_to_master_valid: assert property (slave_accept_to_master_valid) else $display($time," ns",,,"slave_accept_to_master_valid FAIL");

  property no_dual_ready;
    @(posedge clk) disable iff(!reset_n) !(slave_ready&core_ready);
  endproperty
  a_no_dual_ready: assert property (no_dual_ready) else $display($time," ns",,,"no_dual_ready FAIL");

  // property no_ready_after_buffering_stable;
  //   @(posedge clk) disable iff(!reset_n) YOLO;
  // endproperty
  // a_no_ready_after_buffering_stable: assert property (no_ready_after_buffering_stable) else $display($time," ns",,,"no_ready_after_buffering_stable FAIL");

  property data_through_slave;
    logic [RESULT_BITS             -1:0] data_sampled;
    @(posedge clk) disable iff(!reset_n) first_match((slave_accept, data_sampled=slave_data) ##[1:$] master_accept) |-> (master_data === data_sampled);
  endproperty
  a_data_through_slave: assert property (data_through_slave) else $display($time," ns",,,"data_through_slave FAIL");

  property data_through_core;
    logic [RESULT_BITS             -1:0] data_sampled;
    @(posedge clk) disable iff(!reset_n) first_match((core_accept, data_sampled=core_data) ##[1:$] master_accept) |-> (master_data === data_sampled);
  endproperty
  a_data_through_core: assert property (data_through_core) else $display($time," ns",,,"data_through_core FAIL");

endmodule

bind drain drain_property U0 (.*);
