module dual_core_tb
  ();

  import types_pkg::*;


  // Control
  logic                                reset_n;
  logic                                clk;

  // Inputs
  logic [REQUEST_BITS            -1:0] inp_request;
  logic                                inp_valid;
  logic                                inp_ready;

  // Outputs
  logic [RESULT_BITS             -1:0] out_result;
  logic                                out_valid;
  logic                                out_ready;

  localparam NUM_REQS = 10_000;


  dual_core U0 (
    .*
    );

  // bind core_interface core_interface_property U0 (.*);

  initial begin
    clk = 0;
    reset_n = 0;
    inp_ready = 1;
    #4 reset_n = 1;
  end;

  always
    #1 clk = ~clk;


  //*****************************************************************************
  //
  // Save responses from core
  //
  //*****************************************************************************
  int resp_addr = 0;
  reg [RESULT_BITS -1:0] responses [0:NUM_REQS-1];

  always @(posedge clk) begin
    if (inp_ready & out_valid) begin
      responses[resp_addr] <= out_result;
      resp_addr <= resp_addr + 1;
    end
  end


  //*****************************************************************************
  //
  // Load pregenerated requests and send to core
  //
  //*****************************************************************************

  reg [REQUEST_BITS -1:0] requests [0:NUM_REQS-1];

  initial begin
    $readmemb("requests.dat", requests);
  end

  int req_addr;

  assign inp_request = requests[req_addr];

  assign inp_valid = (req_addr < NUM_REQS);

  always_ff @(posedge clk or negedge reset_n) begin
    if (!reset_n) begin
      req_addr <= 0;
    end
    else if (inp_valid & out_ready) begin
      req_addr <= req_addr + 1;
    end
  end

  //*****************************************************************************
  //
  // Save results and stop simulation when all requests has given a response
  //
  //*****************************************************************************

  always_comb begin
    if (resp_addr == NUM_REQS) begin
      $writememb("responses_calc.dat", responses);
      $stop;
    end
  end


endmodule