module fp32_add_tb
  (
  );

  localparam FLOAT32_BITS     = 32;
  localparam FLOAT32_EXP_BITS = 8;

  // Control
  logic                               reset_n;
  logic                               clk;

  // Inputs
  logic [FLOAT32_BITS           -1:0] inp_a;
  logic [FLOAT32_BITS           -1:0] inp_b;
  logic                               inp_op;

  // Outputs
  logic [FLOAT32_BITS           -1:0] out_res;

  fp32_add #(.ROUND(1)) U0 (
    .*
  );

  initial begin
    clk = 0;
    inp_a = 32'h3f800000;
    inp_b = 32'h3f800000;
    inp_op = 1;
  end;

  always  
    #1 clk = ~clk;

  always
    #2 inp_op = ~inp_op;

  always
    #4 inp_a = $urandom();

  always
    #4 inp_b = $urandom();

  shortreal                           v_res_sr;
  logic [FLOAT32_BITS           -1:0] v_res_nxt;
  logic [FLOAT32_BITS           -1:0] v_res_r;

  logic [FLOAT32_EXP_BITS       -1:0] a_exp;
  logic [FLOAT32_EXP_BITS       -1:0] b_exp;
  logic [FLOAT32_EXP_BITS       -1:0] v_res_exp;
  logic [FLOAT32_EXP_BITS       -1:0] v_res_exp_r;


  logic [FLOAT32_BITS           -1:0] a_old;
  logic [FLOAT32_BITS           -1:0] b_old;
  logic                               a_den_r;
  logic                               b_den_r;

  always_comb begin
    v_res_sr = ( inp_op ? $bitstoshortreal(inp_a) - $bitstoshortreal(inp_b)
                        : $bitstoshortreal(inp_a) + $bitstoshortreal(inp_b) );
    v_res_nxt = $shortrealtobits(v_res_sr);
    v_res_exp = v_res_nxt[30:23];
    a_exp = inp_a[30:23];
    b_exp = inp_b[30:23];
    if(v_res_exp == 0) begin
      v_res_nxt = {v_res_nxt[31], 31'd0};
    end
  end



  always_ff @(posedge clk) begin
    assert ((v_res_r == out_res) | (v_res_exp_r == 8'hff) | a_den_r | b_den_r)
        else begin
          $error("Results differ");
          $display("    a=%032b | b= %032b", a_old, b_old);
          $display("    out_res=%032b", out_res);
          $display("      v_res=%032b", v_res_r);
        end
    v_res_r <= v_res_nxt;
    v_res_exp_r <= v_res_exp;
    a_old <= inp_a;
    b_old <= inp_b;
    a_den_r <= (inp_a[30:23] == 0) & (inp_a[22:0] != 0);
    b_den_r <= (inp_b[30:23] == 0) & (inp_b[22:0] != 0);
  end



endmodule
