module fp32_mul_tb
  (
  );

  localparam FLOAT32_BITS     = 32;
  localparam FLOAT32_EXP_BITS = 8;

  // Control
  logic                               reset_n;
  logic                               clk;

  // Inputs
  logic [FLOAT32_BITS           -1:0] inp_a;
  logic [FLOAT32_BITS           -1:0] inp_b;

  // Outputs
  logic [FLOAT32_BITS           -1:0] out_res;

  fp32_mul #(.ROUND(1)) U0 (.*);

  initial begin
    clk = 0;
    inp_a = 32'h3f800000;
    inp_b = 32'h3f800000;
  end;

  always  
    #1 clk = ~clk;

  // always
  //   #2 inp_a = inp_a + 1;

  always
    #2 inp_a = $urandom();

  always
    #2 inp_b = $urandom();

  shortreal                           v_res_sr;
  logic [FLOAT32_BITS           -1:0] v_res_nxt;
  logic [FLOAT32_BITS           -1:0] v_res_r;

  logic [FLOAT32_EXP_BITS       -1:0] a_exp;
  logic [FLOAT32_EXP_BITS       -1:0] b_exp;
  logic [FLOAT32_EXP_BITS       -1:0] v_res_exp;
  logic [FLOAT32_EXP_BITS       -1:0] v_res_exp_r;


  always_comb begin
    v_res_sr = $bitstoshortreal(inp_a) * $bitstoshortreal(inp_b);
    v_res_nxt = $shortrealtobits(v_res_sr);
    a_exp = inp_a[30:23];
    b_exp = inp_b[30:23];
    v_res_exp = v_res_nxt[30:23];
    if((v_res_exp == 0) | (a_exp == 0) | (b_exp == 0)) begin
      v_res_nxt = {v_res_nxt[31], 31'd0};
    end
  end

  always_ff @(posedge clk) begin
    assert ((v_res_r == out_res) | (v_res_exp_r == 8'hff))
        else begin
          $error("Results differ");
          $display("    out_res=%032b", out_res);
          $display("      v_res=%032b", v_res_r);
        end
    v_res_r <= v_res_nxt;
    v_res_exp_r <= v_res_exp;
  end



endmodule
