module ray_datapath_tb ();

  import types_pkg::*;

  logic                                reset_n;
  logic                                clk;

  // Inputs
  logic [WORD_BITS               -1:0] inp_req;
  command_t                            inp_command;
  logic [WORD_BITS               -1:0] inp_obj_data;
  instruction_union_t                  inp_instruction;

  // Outputs
  logic                                out_req_shift;
  logic [BYTE_BITS               -1:0] out_result;
  logic                                out_result_valid;
  logic [$clog2(THREADS)         -1:0] out_thread;
  status_t                             out_status;
  logic                                out_status_valid;
  logic [OBJ_BUFFER_ADDR_BITS    -1:0] out_obj_addr;
  logic                                out_obj_read;
  logic [PC_BITS                 -1:0] out_instruction_address;
  


  ray_datapath U0 ( 
  .*
  ); 
    
  initial begin
    clk = 0;
    reset_n = 0;
    #4 reset_n = 1;
  end;


  always  
    #1 clk = ~clk;

  always_comb
    inp_command = (out_thread == 2'b10) ? RESBUF_GNT : NONE;


  instruction_memory IMEM (
    .clk(clk),
    .inp_addr(out_instruction_address),
    .out_instruction(inp_instruction)
  );



  reg [WORD_BITS -1:0] objbuf [0:2**OBJ_BUFFER_ADDR_BITS -1];

  initial begin
    $readmemh("objbuffer.dat", objbuf);
  end

  always_ff @(posedge clk) begin
    if (out_obj_read) begin
      inp_obj_data <= objbuf[out_obj_addr];
    end
  end

  logic [3*WORD_BITS -1:0] reqbuf = {32'h3b98cbdf, 32'hbb98cbdf, 32'hbf7ffe93};

  assign inp_req = reqbuf[WORD_BITS -1:0];

  always_ff @(posedge clk) begin
    if (out_req_shift) begin
      reqbuf <= {32'h00000000, reqbuf[3*WORD_BITS -1:WORD_BITS]};
    end
  end

  logic [3*BYTE_BITS -1:0] resbuf;

  always_ff @(posedge clk) begin
    if (out_result_valid) begin
      resbuf <= {out_result, resbuf[3*BYTE_BITS -1:BYTE_BITS]};
    end
  end


// Ray direction:
// x: 32'h3b98cbdf
// y: 32'hbb98cbdf
// z: 32'hbf7ffe93

// Calculated colour from python model:
// r: 8'h00
// g: 8'h62
// b: 8'h00


endmodule