module reverse_regslice_property
  (
    // Outputs
    slave_ready, master_valid, master_data,
    // Inputs
    clk, reset_n, slave_valid, master_ready, slave_data
  );

  parameter DATA_WIDTH = 8;

  import types_pkg::*;

  // Control
  input  logic                                reset_n;
  input  logic                                clk;

  // Slave interface
  input  logic                                slave_valid;
  input  logic                                slave_ready;
  input  logic [DATA_WIDTH              -1:0] slave_data;

  // Master interface
  input  logic                                master_valid;
  input  logic                                master_ready;
  input  logic [DATA_WIDTH              -1:0] master_data;


  logic                                       slave_accept;
  logic                                       master_accept;
  assign slave_accept = slave_ready & slave_valid;
  assign master_accept = master_ready & master_valid;

  property master_data_stable;
    @(posedge clk) disable iff(!reset_n) (master_valid & !master_ready) |=> $stable(master_data);
  endproperty
  a_master_data_stable: assert property (master_data_stable) else $display($time," ns",,,"master_data_stable FAIL");

  property master_valid_stable;
    @(posedge clk) disable iff(!reset_n) (master_valid & !master_ready) |=> master_valid;
  endproperty
  a_master_valid_stable: assert property (master_valid_stable) else $display($time," ns",,,"master_valid_stable FAIL");

  property slave_accept_to_master_valid;
    @(posedge clk) disable iff(!reset_n) (slave_accept) |-> master_valid;
  endproperty
  a_slave_accept_to_master_valid: assert property (slave_accept_to_master_valid) else $display($time," ns",,,"slave_accept_to_master_valid FAIL");

  property no_ready_after_buffering;
    @(posedge clk) disable iff(!reset_n) (slave_accept & !master_ready) |=> !slave_ready;
  endproperty
  a_no_ready_after_buffering: assert property (no_ready_after_buffering) else $display($time," ns",,,"no_ready_after_buffering FAIL");

  // property no_ready_after_buffering_stable;
  //   @(posedge clk) disable iff(!reset_n) YOLO;
  // endproperty
  // a_no_ready_after_buffering_stable: assert property (no_ready_after_buffering_stable) else $display($time," ns",,,"no_ready_after_buffering_stable FAIL");

  property data_through;
    logic [DATA_WIDTH              -1:0] data_sampled;
    @(posedge clk) disable iff(!reset_n) first_match((slave_accept, data_sampled=slave_data) ##[0:$] master_accept) |-> (master_data === data_sampled);
  endproperty
  a_data_through: assert property (data_through) else $display($time," ns",,,"data_through FAIL");

endmodule

bind reverse_regslice reverse_regslice_property U0 (.*);
